@@ -20,6 +20,279 @@ import matplotlib.gridspec as gridspec
 from matplotlib import animation
 from scipy import interpolate
 
+def read_phonopy(file_eigenvectors, n_atom_unit_cell):
+    ## =============================================================================
+    # Phonopy frequencies and eigenvectors
+    data = yaml.load(open(file_eigenvectors))
+    #D = data['phonon'][0]['dynamical_matrix']
+    #D = np.array(D)
+    #D_real, D_imag = D[:,0::2], 1j*D[:,1::2]
+    #D = (D_real + D_imag)*21.49068**2#*0.964*10**(4)#
+    
+#    data2 = data['phonon']
+#    qpoints_scaled = []
+#    freqs = []
+#    eigvecs = []
+#    for element in data2:
+#        qpoints_scaled.append(element['q-position'])
+#        freq = []
+#        eigvec = np.zeros((n_atom_unit_cell*3, n_atom_unit_cell*3),dtype=complex)
+#        for j in range(len(element['band'])):
+#            branch = element['band'][j]
+#            freq.append(branch['frequency'])
+#            
+#            eigen = np.array(branch['eigenvector'],dtype=complex)
+#            eigen_real = eigen[:,:,0]
+#            eigen_imag = eigen[:,:,1]
+#            eigen = eigen_real + 1j*eigen_imag
+#            eigen = eigen.reshape(n_atom_unit_cell*3,)
+#            eigvec[:,j] = eigen
+#    
+#        freqs.append(freq)
+#        eigvecs.append(eigvec)
+#    qpoints_scaled = np.array(qpoints_scaled)
+#    freqs = np.array(freqs)
+    c = 1.88973 #conversion to Bohrs
+    Hk = np.array(data['reciprocal_lattice'])*2*np.pi*1/c
+    distances = []
+    qpoints_scaled = []
+    dist = 0.0
+    frequencies = []
+    eigvecs = []
+    for i in range(len(data['phonon'])):
+        if(i==0):
+            this_element = data['phonon'][i]
+            this_qpoint_sc = this_element['q-position']
+            dist = 0.0
+            distances.append(dist)
+        else:
+            this_element = data['phonon'][i]
+            previous_element = data['phonon'][i-1]
+            this_qpoint_sc = this_element['q-position']
+            previous_qpoint_sc =  previous_element['q-position']
+            
+            this_qpoint = np.dot(Hk,this_qpoint_sc)
+            previous_qpoint = np.dot(Hk,previous_qpoint_sc)
+            
+            diff = this_qpoint - previous_qpoint
+            
+            dist = dist + np.linalg.norm(diff)
+            distances.append(dist)
+        
+        prov = []
+        eigvec = np.zeros((n_atom_unit_cell*3, n_atom_unit_cell*3),dtype=complex)
+        for j in range(len(this_element['band'])):
+            branch = this_element['band'][j]
+            prov.append(branch['frequency'])
+            eigen = np.array(branch['eigenvector'],dtype=complex)
+            eigen_real = eigen[:,:,0]
+            eigen_imag = eigen[:,:,1]
+            eigen = eigen_real + 1j*eigen_imag
+            eigen = eigen.reshape(n_atom_unit_cell*3,)
+            eigvec[:,j] = eigen
+            
+        eigvecs.append(eigvec)
+        frequencies.append(prov)
+        qpoints_scaled.append(this_qpoint_sc)
+
+    frequencies = np.array(frequencies)
+    distances = np.array(distances)
+    qpoints_scaled = np.array(qpoints_scaled)
+    Nqpoints = len(qpoints_scaled[:,0])
+
+    
+    ks = np.dot(Hk,qpoints_scaled.T).T
+    # =============================================================================
+    return Nqpoints, qpoints_scaled, ks, frequencies, eigvecs, distances, Hk
+
+def aaa_interpol_spectrum(ks_path, freq, freqs_from_spectrum, eigvecs, modes, ZQS, new_ks_in_between):
+    num_ks = len(ks_path)
+    num_modes = len(freqs_from_spectrum[0,:])
+    num_freqs = len(ZQS[:,0,0])
+    kk = np.linspace(0,1,num_ks)
+    num_Gammas, indexes_Gammas, indexes_Gammas_withboundaries = find_Gammas(ks_path)
+    df = freq[1] - freq[0]
+    X, Y = np.meshgrid(kk,freq)
+    
+#    get_Force_Constants(eigvecs, freqs_from_spectrum.T)
+    interp_freq = []
+    INTERP = []
+    for j in range(len(modes)):
+        interp_freq_interm = np.array([])
+        for m in range(len(indexes_Gammas)):
+            this_G = indexes_Gammas_withboundaries[m]
+            next_G = indexes_Gammas_withboundaries[m+1]+1
+            kk_i = kk[this_G:next_G]
+            f_i = freqs_from_spectrum[this_G:next_G,j]
+            interp = interp1d(kk_i, f_i, 'cubic')
+#            x_i = np.arange(kk_i[0], kk_i[-1], 0.001)
+            INTERP.append(interp)
+#            interp_freq_interm = np.concatenate((interp_freq_interm, interp(x_i)))
+#        interp_freq_interm = np.array(interp_freq_interm)
+#        interp_freq.append(interp_freq_interm)
+#    interp_freq = np.array(interp_freq)
+
+    ZQS_interp = np.zeros((num_freqs,num_modes, new_ks_in_between*(num_ks-1)+num_ks))
+    
+    
+    ktot = 0
+    for k in range(num_ks-1):
+        sector = 0
+        for l in indexes_Gammas_withboundaries[1::]:
+            if(k >= l):
+                sector = sector + 1
+        count_k_in_between = 0
+        for j in range(num_modes):  
+            this_gaussian = ZQS[:,j,k]
+            next_gaussian = ZQS[:,j,k+1]
+            
+#            import matplotlib.pyplot as plt
+#            fig,ax = plt.subplots()
+#            #to comment
+#            popt_this, pcov_this = curve_fit(gaussian, freq, this_gaussian)
+#            popt_next, pcov_next = curve_fit(gaussian, freq, next_gaussian)
+            
+            this_max = freqs_from_spectrum[k,j]
+            next_max = freqs_from_spectrum[k+1,j]
+            diff_maxima = next_max - this_max
+            
+            if(diff_maxima >= 0):
+                this_gaussian_interp = interp1d(freq, this_gaussian, bounds_error=False, fill_value=0)
+                next_gaussian_interp = interp1d(freq, next_gaussian, bounds_error=False, fill_value=0)
+                this_gaussian_fit = this_gaussian_interp(freq-diff_maxima)
+                next_gaussian_fit = next_gaussian_interp(freq)
+                shift_interp = -next_max
+            else:
+                this_gaussian_interp = interp1d(freq, this_gaussian, bounds_error=False, fill_value=0)
+                next_gaussian_interp = interp1d(freq, next_gaussian, bounds_error=False, fill_value=0)
+                this_gaussian_fit = this_gaussian_interp(freq)
+                next_gaussian_fit = next_gaussian_interp(freq+diff_maxima)
+                shift_interp = -this_max
+            
+#            area_this_gaussian = np.trapz(this_gaussian_fit, freq)
+#            area_next_gaussian = np.trapz(next_gaussian_fit, freq)
+
+            this_k = kk[k]
+            next_k = kk[k+1]
+            interm_k = np.linspace(this_k, next_k, new_ks_in_between+2)[0:-1]
+
+            interp = INTERP[sector+j*num_Gammas]
+            intermediate_maxima = interp(interm_k)
+            
+            count_k_in_between = 0
+            for m in range(new_ks_in_between+1):
+                mi = intermediate_maxima[m]
+                x = (interm_k[m] - interm_k[0])/(next_k-interm_k[0])
+#                area_factor = area_this_gaussian+(area_next_gaussian-area_this_gaussian)*x
+                interp_gaussian = this_gaussian_fit+(next_gaussian_fit-this_gaussian_fit)*x
+                interp_gaussian_interp = interp1d(freq, interp_gaussian,bounds_error=False, fill_value=0)
+                interp_gaussian_fit = interp_gaussian_interp(freq-(shift_interp+mi))
+                ZQS_interp[:,j,ktot+count_k_in_between] = interp_gaussian_fit
+                count_k_in_between = count_k_in_between + 1
+                
+
+#                popt = popt_this + (popt_next-popt_this)*x
+#                interp_gaussian_fit = gaussian(freq, *popt)#1/(sig*np.sqrt(2*np.pi)) * np.exp(-.5*((freq-mi)/sig)**2) *area_factor
+
+#                ax.plot(freq, interp_gaussian_fit, label='interp')
+#            ax.plot(freq, this_gaussian, label='real this')
+#            ax.plot(freq,next_gaussian, label='real next')
+##            ax.plot(freq, this_gaussian_fit, label='real this fit')
+##            ax.plot(freq,next_gaussian_fit, label='real next fit')
+#            ax.legend()
+            
+        ktot = ktot + count_k_in_between
+    ZQS_interp[:,:,-1] = ZQS[:,:,-1]
+    return ZQS_interp
+
+def aaa_get_real_branches(freqs, eigvecs, masses, Hk, qpoints_scaled):
+    num_ks = len(freqs[0,:])
+    num_branches = len(freqs[:,0])    
+    
+    freqs_until_now = freqs[:,0]
+    indexes = np.tile(np.arange(num_branches), (2,1))
+    new_indexes = np.arange(num_branches)
+    for k in range(1,num_ks-1):
+        kpoint = np.dot(Hk, qpoints_scaled[k])
+        kpoint = kpoint/np.linalg.norm(kpoint)
+        next_kpoint = np.dot(Hk, qpoints_scaled[k+1])
+        if(np.allclose(next_kpoint,[0,0,0])):
+            next_kpoint = [0,0,0]
+        else:
+            next_kpoint = next_kpoint/np.linalg.norm(next_kpoint)
+#        print(k,qpoints_scaled[k], np.round(next_kpoint,2))
+
+        this_eigvecs = eigvecs[k].T/np.sqrt(masses)*10
+        next_eigvecs = eigvecs[k+1].T/np.sqrt(masses)*10
+        next_eigvecs = next_eigvecs/np.linalg.norm(next_eigvecs, axis=0)
+
+#        print(np.round(next_eigvecs[0::],2))
+
+        temp = new_indexes#np.arange(num_branches)
+        for j,l in zip(range(num_branches), new_indexes):
+            eig = this_eigvecs[l,:]
+            eig = eig/np.linalg.norm(eig)
+#            print(l,np.round(eig,2), freqs[l,k])
+            dot_prod = np.abs(np.dot(next_eigvecs,eig))
+#            print(np.round(dot_prod,2))
+
+
+            new_index = np.argwhere([dot_prod.max()-0.0000000000001 <= dd <= dot_prod.max() for dd in dot_prod])
+            degenerate = len(new_index)
+#            print(new_index)
+            if (len(new_index) > 1):
+#                print('two are degenerate')
+#                next_eigvecs_deg = next_eigvecs[new_index,:]
+#                dot_nexteigvecs_nextk = np.dot(next_eigvecs_deg.reshape(degenerate*2,3), next_kpoint)
+#                dot_nexteigvecs_nextk = np.abs(dot_nexteigvecs_nextk.reshape(degenerate,2))
+#                dot_nexteigvecs_nextk = dot_nexteigvecs_nextk/np.linalg.norm(dot_nexteigvecs_nextk)
+#                dot_thiseig_nextk = np.abs(np.dot(eig.reshape(1*2,3), next_kpoint))
+#                dot_thiseig_nextk = dot_thiseig_nextk/np.linalg.norm(dot_thiseig_nextk)
+#                diff = np.abs(dot_thiseig_nextk - dot_nexteigvecs_nextk)
+#                diff_sum = np.sum(diff, axis=1)
+#                print(np.round(dot_nexteigvecs_nextk,2))
+#                print(np.round(dot_thiseig_nextk,2))
+#                print('diffsum', np.round(diff_sum,2))
+                a=0
+                if(a==10):#(np.isclose(dot_nexteigvecs_nextk, 1,  rtol=0.2, atol=0.2).any()):
+                    #you have a longitudinal branch
+                    print('theres a longitudinal')
+#                    isclose = np.isclose(dot_nexteigvecs_nextk, dot_thiseig_nextk, rtol=0.2, atol=.2)
+#                    true_long = np.all(isclose, axis=1)
+#                    arg_long = np.argwhere(true_long)
+#                    new_index = new_index[arg_long]
+#                    print(new_index)
+#                    
+                else:
+#                    c = np.argwhere(diff_sum==diff_sum.min())
+                    new_index = l#new_index[c]
+                    print(new_index)
+
+                
+                
+            if (new_index in temp[0:j]): #it happens at Gamma where TA modes are degenerate
+#                print('aaaa')
+                new_index = l
+
+            temp[j] = new_index
+#            print()
+        new_indexes = temp.astype(int)
+#        print(new_indexes)
+#        print()
+
+
+
+        indexes = np.vstack((indexes,new_indexes))
+        
+    indexes = indexes.astype(int)
+#    print(indexes)
+    freqs_sorted = []
+    for j in range(num_ks):
+        freqs_sorted.append(freqs[indexes[j],j])   
+        
+    return indexes, np.array(freqs_sorted).T
+
+
 def plot4_old(kk, frequencies, frequencies_disp, qpoints, x_labels, ZQS):
     """
     old plot 4, with mathematical interpolation
