@@ -189,12 +189,35 @@ def get_kpath_many(kinput, Hk, npoints=20):
     return ks_scal, ks, distances
 
 def get_kgrid(N1k, N2k, N3k, Hk):
-    x,y,z = np.linspace(0,1,N1k+1)[0:-1]-0.5, np.linspace(0,1,N2k+1)[0:-1]-0.5, np.linspace(0,1,N3k+1)[0:-1]-0.5
+    x,y,z = np.linspace(0,1,N1k+1, endpoint=True)[0:-1], np.linspace(0,1,N2k+1, endpoint=True)[0:-1], np.linspace(0,1,N3k+1, endpoint=True)[0:-1]
     kpoints_scal = np.array(np.meshgrid(x,y,z)).T.reshape(-1,3)
     kpoints = np.dot(Hk,kpoints_scal.T).T
-    return kpoints_scal, kpoints
+    Nqs = len(kpoints)
+    return Nqs, kpoints_scal, kpoints
 
-def check_is_symm(k_scal, ks_scal_path):
+def get_kgrid_shifted(N1k, N2k, N3k, Hk):
+    x,y,z = np.linspace(0,1,N1k+1, endpoint=True)[0:-1]-0.5, np.linspace(0,1,N2k+1, endpoint=True)[0:-1]-0.5, np.linspace(0,1,N3k+1, endpoint=True)[0:-1]-0.5
+    kpoints_scal = np.array(np.meshgrid(x,y,z)).T.reshape(-1,3)
+    kpoints = np.dot(Hk,kpoints_scal.T).T
+    Nqs = len(kpoints)
+    return Nqs, kpoints_scal, kpoints
+
+def check_outside_BZ(k_scal, ks_scal_path):
+    accepted_ks = []
+
+    #second check: is this irr k in the k path outside the 1st BZ?
+    for s in range(0,3): #3 è un numero a caso, se l'utente va oltre 2 BZ di distanza è proprio un coglione!
+        shift = np.eye(3)*s
+        ks_outside_BZ = shift + k_scal
+        for k in range(3):
+            k_outside_BZ = ks_outside_BZ[k,:]
+            if (np.any(np.all(np.isclose(k_outside_BZ-ks_scal_path,0), axis=1))):
+                accepted_ks.append(k_outside_BZ)
+            
+    accepted_ks = np.array(accepted_ks)[2:]  
+    return accepted_ks
+
+def check_is_symm_irreducible(k_scal, ks_scal_path):
     permutations = np.array([[[0,1,0],[1,0,0],[0,0,1]], 
                              [[0,0,1],[0,1,0],[1,0,0]], 
                              [[1,0,0],[0,0,1],[0,1,0]],
@@ -268,8 +291,9 @@ def get_irr_ks(ks, ks_scaled):
     
     irr_ks_scal = ks_scaled[index_irr_ks,:]
     irr_ks = ks[index_irr_ks,:]
+    N_irr_qs = len(irr_ks)
     
-    return irr_ks, irr_ks_scal, np.array(weights), permutations_types
+    return N_irr_qs, irr_ks, irr_ks_scal, np.array(weights), permutations_types
 
 
 def get_eigvals_eigvec(D):
